import React, { useCallback, useContext, useEffect } from "react";
import { Typography, Link as MuiLink, makeStyles } from "@material-ui/core";
import { SimpleAccordion } from "../common/Accordion/SimpleAccordion";
import { IOCDetails } from "./IOCDetails";
import { Link } from 'react-router-dom';
import { LokiContainer } from "../common/Loki/LokiContainer";
import GitRefLink from "./GitRefLink";
import AlertMessages from "./AlertMessages";
import { IOCService } from "./IOCService";
import { IOCCommandTable } from "./IOCCommandTable";
import AccessControl from '../../components/auth/AccessControl';
import { useDeploymentJobById, userContext } from "../../api/SwaggerApi";
import { DeploymentStatus, IocActiveDeployment } from "../../api/DataTypes";
import { theme } from "../../style/Theme";

const useStyles = makeStyles((theme) => ({
  iocNotDeployed: {
    color: theme.iocNotDeployed.color
  }
}));

export function IOCLiveStatus({ ioc, currentCommand, commands, getCommands, buttonDisabled, setButtonDisabled,
  commandLazyParams, setCommandLazyParams, commandColumnSort, setCommandColumnSort, rowsPerPage }) {
  const { user } = useContext(userContext);
  const [deploymentJob, getDeploymentJobById] = useDeploymentJobById();

  const classes = useStyles(theme);

  useEffect(() => {
    if (ioc.activeDeployment?.awxJobId) {
      getDeploymentJobById(ioc.activeDeployment?.awxJobId);
    }
  }, [getDeploymentJobById, ioc.activeDeployment?.awxJobId])

  const liveIOC = { ...ioc };
  liveIOC.name = ioc.namingName;

  const getSubset = useCallback((ioc) => {

    const iocHascDeployment = new IocActiveDeployment(ioc.activeDeployment).hasDeployment();
    
    //if IOC is not deployed some fields has to show '---' value
    if (!iocHascDeployment) {
      var notDeployedText = {shownValue: '---', shownClass: classes.iocNotDeployed};
    }
    

    //Show START/STOP components only when IOC was deployed SUCCESSFULLY
    const deploymentStatus = new DeploymentStatus(ioc.activeDeployment, deploymentJob);
    const showControls = deploymentStatus.wasSuccessful();
    let subset = {
      "IOC name":
        <Typography><MuiLink href={`${window.CCDB_ADDRESS}?name=${ioc.namingName}`}
          target="_blank" rel="noreferrer">{ioc.namingName}</MuiLink></Typography>,
      "Description": ioc.description,
      "Owner": ioc.owner,
      "Git reference": iocHascDeployment ? <GitRefLink ioc={ioc} /> : notDeployedText,
      "Deployed on": iocHascDeployment ?
        <Typography>
          {ioc.activeDeployment?.host.csEntryId ?
            <Link to={`/hosts/${ioc.activeDeployment?.host.csEntryId}`}>{ioc.activeDeployment?.host.fqdn}</Link>
            :
            <>{ioc.activeDeployment?.host.fqdn}</>
          }
        </Typography> 
        : notDeployedText,
    }
    if (user) {
    subset["IOC Service Controls"] = showControls ?
      <IOCService {...{ ioc, currentCommand, getCommands, buttonDisabled, setButtonDisabled, commandColumnSort, commandLazyParams }} />
      : "Can't be controlled, because IOC is not (yet) deployed succesfully";
  }

  return subset
}, [buttonDisabled, commandColumnSort, commandLazyParams, currentCommand, deploymentJob, getCommands, setButtonDisabled, user, classes.iocNotDeployed]);

function isIocDeployed(ioc) {
  return Boolean(ioc.activeDeployment);
}

return (
  <>
    <IOCDetails ioc={liveIOC} getSubset={getSubset} alert={<AlertMessages alerts={liveIOC.alerts} />} />
    <SimpleAccordion summary="IOC Service Control log">
      <IOCCommandTable commands={commands}
        commandLazyParams={commandLazyParams} setCommandLazyParams={setCommandLazyParams} commandColumnSort={commandColumnSort} setCommandColumnSort={setCommandColumnSort}
        rowsPerPage={rowsPerPage} />
    </SimpleAccordion>
    {(liveIOC.activeDeployment?.host?.csEntryIdValid) &&
      <AccessControl allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}>
        <SimpleAccordion summary="ProcServLog info" defaultExpanded>
          <LokiContainer csEntryId={liveIOC.activeDeployment?.host.csEntryId} iocName={ioc.namingName} isDeployed={isIocDeployed(ioc)} />
        </SimpleAccordion>
      </AccessControl>
    }
  </>
);
}
