import { Button, LinearProgress, Grid, Typography, Tooltip } from "@material-ui/core";
import { makeStyles } from '@material-ui/core/styles';
import React, { useState, useEffect, useRef, useContext } from "react";
import { useStartIOC, useStopIOC } from "../../api/SwaggerApi";
import { CommandJobStepper } from "./CommandJobStepper";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import { ConfirmationDialog } from "../dialog/ConfirmationDialog";
import { notificationContext } from "../../components/common/notification/Notifications";
import { Alert } from "@material-ui/lab";
import { theme } from "../../style/Theme";
import { initRequestParams } from "../common/Helper";
import AccessControl from "../auth/AccessControl";
import { essColors } from "../../style/Palette";

const useStyles = makeStyles({
  startButton: {
    backgroundColor: essColors.grass,
    color: '#FFFFFF',
    "&:disabled": {
      backgroundColor: '#e0e0e0',
      color: '#afaba6',
    },
    "&:hover": {
      backgroundColor: essColors.grass_op,
      color: '#FFFFFF',
    }
  },
  stopButton: {
    backgroundColor: essColors.navy,
    color: '#FFFFFF',
    "&:disabled": {
      backgroundColor: '#e0e0e0',
      color: '#afaba6',
    },
    "&:hover": {
      backgroundColor: essColors.navy_op,
      color: '#FFFFFF',
    }
  },
});

export function IOCService({ ioc, currentCommand, getCommands, buttonDisabled, setButtonDisabled, commandLazyParams, commandColumnSort }) {
  const classes = useStyles(theme);
  const [error, setError] = useState();

  function onError(message) {
    setError(message);
  }

  const [startJob, startIOC, resetStartJob] = useStartIOC(ioc.id, onError);
  const [stopJob, stopIOC, resetStopJob] = useStopIOC(ioc.id, onError);
  const [inProgress, setInProgress] = useState(false);
  const [action, setAction] = useState(false);
  const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
  const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
  const [adHocDialogDescription, setAdHocDialogDescription] = useState();
  const [command, setCommand] = useState(null);
  const callbackRef = useRef();
  const { watchCommand } = useContext(notificationContext);

  useEffect(() => {
    if (startJob && (!command || command.id !== startJob.id)) {
      watchCommand(startJob.id);
      setCommand(startJob);
      setInProgress(false);
    }
    else if (stopJob && (!command || command.id !== stopJob.id)) {
      watchCommand(stopJob.id);
      setCommand(stopJob);
      setInProgress(false);
    }
    else if (currentCommand) {
      watchCommand(currentCommand.id);
      setCommand(currentCommand);
      setInProgress(false);
    }
  }, [startJob, stopJob, command, watchCommand, getCommands, currentCommand]);

  function resetUI() {
    setCommand(null);
    setAction(null);
    resetStartJob();
    resetStopJob();
    setError(null);
  }

  const start = async () => {
    resetUI();
    setAction("Start");
    setInProgress(true);
    setButtonDisabled(true);
    startIOC();
    let requestParams = initRequestParams(commandLazyParams, null, commandColumnSort);
    requestParams.deployment_id = ioc.activeDeployment?.id;
    getCommands(requestParams);
  }

  const stop = async () => {
    resetUI();
    setAction("Stop");
    setInProgress(true);
    setButtonDisabled(true);
    await stopIOC();

    let requestParams = initRequestParams(commandLazyParams, null, commandColumnSort);

    requestParams.deployment_id = ioc.activeDeployment?.id;
    getCommands(requestParams);
  }

  const openStartModal = () => {
    setAdHocDiatlogTitle("Starting IOC");
    setAdHocDialogDescription(
      <>
        <Typography style={{ display: 'inline-block' }}>Are you sure want to start &nbsp;</Typography>
        <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
      </>);
    callbackRef.current = start;
    setAdHocDialogOpen(true);
  }

  const openStopModal = () => {
    setAdHocDiatlogTitle("Stopping IOC");
    setAdHocDialogDescription(
      <>
        <Typography style={{ display: 'inline-block' }}>Are you sure want to stop &nbsp;</Typography>
        <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
      </>);
    callbackRef.current = stop;
    setAdHocDialogOpen(true);
  }

  let disabledStartButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStartButtonTitle = "There is an ongoing operation, you can not Start the IOC right now";
  }

  let disabledStopButtonTitle = "";
  if (buttonDisabled || ioc.operationInProgress) {
    disabledStopButtonTitle = "There is an ongoing operation, you can not Stop the IOC right now";
  }

  return (
    <>
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => "You are not authorized to control this IOC"}>
        <SimpleModal open={adHocDialogOpen} setOpen={setAdHocDialogOpen}>
          <ConfirmationDialog open={adHocDialogOpen} setOpen={setAdHocDialogOpen} title={adHocDialogTitle} description={adHocDialogDescription} callback={callbackRef.current} />
        </SimpleModal>

        <Grid container spacing={1}>
          <Grid item xs={1} md={1}>
            <Tooltip title={disabledStopButtonTitle}>
              <span>
                <Button className={classes.stopButton} onClick={openStopModal} disabled={buttonDisabled}>Stop</Button>
              </span>
            </Tooltip>
          </Grid>
          <Grid item xs={1} md={1}>
            <Tooltip title={disabledStartButtonTitle}>
              <span>
                <Button className={classes.startButton} onClick={openStartModal} disabled={buttonDisabled}>Start</Button>
              </span>
            </Tooltip>
          </Grid>
          <Grid item xs={12} md={12}>
            {error ?
              <Alert severity="error">{error}</Alert>
              :
              <>
                {command ?
                  <CommandJobStepper awxCommandId={command.awxCommandId} actionType={action} /> : inProgress ? <LinearProgress color="primary" /> : <></>
                }
              </>
            }
          </Grid>
        </Grid>
      </AccessControl>
    </>
  )
}
